from parser_base import ParsingError, BaseParser


class Parser(BaseParser):
    """
    Вычисления производятся в процессе разбора

    Грамматика:
        num -> <число>
        group -> num | '(' add ')'
        mult -> group ( ('*' | '/') group )*
        add -> mult ( ('+' | '-') mult )*
        start -> add
    """

    def __init__(self, text: str):
        """Конструктор
        :param text: текст программы
        """
        super().__init__(text)
        self.vars = {}

    def num(self) -> float:
        """num   -> <число>"""
        self._ws()
        pos = self.pos
        while self._curr.isdecimal() or self._curr == '.':
            self.pos += 1
        res = float(self.text[pos:self.pos])
        self._ws()
        return res

    def group(self) -> float:
        """group -> num | '(' add ')'"""
        if self._is_parse('('):
            self._parse('(')
            res = self.add()
            self._parse(')')
            return res
        elif self._curr.isdecimal() or self._curr == '.':
            return self.num()
        else:
            raise ParserError(f'Недопустимый символ {self._curr} в позиции {self._pos}')

    def mult(self) -> float:
        """mult  -> group (('*' | '/') group)*"""
        res = self.group()
        while self._is_parse('*', '/'):
            op = self._parse('*', '/')
            tmp = self.group()
            res = res * tmp if op == '*' else res / tmp
        return res

    def add(self) -> float:
        """add   -> mult (('+' | '-') mult)*"""
        res = self.mult()
        while self._is_parse('+', '-'):
            op = self._parse('+', '-')
            tmp = self.mult()
            res = res + tmp if op == '+' else res - tmp
        return res

    def start(self) -> float:
        return self.add()
