from contextlib import suppress
from typing import Union


class ParsingError(Exception):
    """Класс для исключений парсинга"""

    def __init__(self, message):
        super().__init__(message)


class BaseParser:
    """Базовый класс для построения парсеров"""

    def __init__(self, text: str):
        """Конструктор
        :param text: текст программы
        """
        self.text = text + '$'
        self.pos = 0

    @property
    def _curr(self) -> str:
        """Возвращает текущий символ"""
        return self.text[self.pos]

    def _ws(self):
        """Пропускает пробельные символы"""
        while self._curr.isspace():
            self.pos += 1

    def _parse(self, s1: str, *s: str) -> Union[str, None]:
        """Ищет в текущей позиции и возвращает первое совпадение из (s1, *s)
        Указатель смещается
        :param (s1, *s) - допустимые фрагменты
        :exception ParsingError
        """
        self._ws()
        # for ss in (s1, *s):
        for ss in (s1,) + s:
            if self.text[self.pos:self.pos + len(ss)] == ss:
                self.pos += len(ss)
                self._ws()
                return ss
        # raise ParsingError("Ни один фрагмент из {} не найден!".format((s1, *s)))
        raise ParsingError("Ни один фрагмент из {} не найден!".format((s1,) + s))

    def _is_parse(self, s1: str, *s: str) -> Union[str, None]:
        """Ищет в текущей позиции и возвращает первое совпадение из (s1, *s)
        Указатель не смещается
        :param (s1, *s) - допустимые фрагменты
        """
        pos = self.pos
        res = None
        with suppress(ParsingError):
            res = self._parse(s1, *s)
        self.pos = pos
        return res
